## ----setup, include=FALSE---------------------
knitr::opts_chunk$set(echo = TRUE,
                      error = TRUE,
                      warning = FALSE,
                      message = FALSE,
                      cache = TRUE,
                      fig.align = "center")


## ---------------------------------------------
#install.packages("gamlss")
library(gamlss)#load gamlss
#?gamlss
#?`gamlss-package`
#?`gamlss.dist-package`
#?`gamlss.dist-package`
#?gamlss.family


## ---- eval=FALSE------------------------------
## gamlss(formula = formula(data),
##        sigma.formula = ~1,
##        nu.formula = ~1,
##        tau.formula = ~1,
##        family = NO(),
##        data,
##        weights = NULL,
##        contrasts = NULL,
##        method = RS(),
##        start.from = NULL,
##        mu.start = NULL,
##        sigma.start = NULL,
##        nu.start = NULL,
##        tau.start = NULL,
##        mu.fix = FALSE,
##        sigma.fix = FALSE,
##        nu.fix = FALSE,
##        tau.fix = FALSE,
##        control = gamlss.control(...),
##        i.control = glim.control(...), ...)


## ---------------------------------------------
# pb() and ps() functions
data(aids)
#?aids

library(ggplot2)
ggplot(aids,aes(x,y))+
  geom_point()+
  geom_smooth(method = "glm",
              formula = y~x,
              se=FALSE,
              method.args = list(family = poisson(link="log")))


#pb(x, df = NULL, lambda = NULL, max.df=NULL,control = pb.control(...), ...)
#pb(), the current version of P-splines which uses SVD in the fitting and therefore is the most reliable

PO()
show.link(family = "PO")

aids1<-gamlss(y~pb(x, df=10),
              data=aids,
              family=PO(mu.link = "log"),
              control = gamlss.control(trace = FALSE)) 

aids2<-gamlss(y~pb(x, df=3),
              data=aids,
              family=PO(mu.link = "log"),
              control = gamlss.control(trace = FALSE)) 

aids3<-gamlss(y~pb(x),
              data=aids,
              family=PO(mu.link = "log"),
              control = gamlss.control(trace = TRUE)) 

summary(aids1)#df=10 and edf(aids1)=12
#summary(aids2)#df=3 and edf(aids2)=4.999993
#summary(aids3)#edf(aids3)=6.450721


## ---------------------------------------------
aids$predicted <- aids1$mu.fv

ggplot(aids,aes(x,y))+
  geom_point()+
  geom_smooth(method = "glm",
              formula = y~x,
              se=FALSE,
              method.args = list(family = poisson(link="log")))+
geom_smooth(data=aids,aes(y=predicted),
            col="red",
            stat="identity")


## ---------------------------------------------
## Some details about the gamlss output for aids dataset
names(aids1)


## ---- eval=FALSE------------------------------
## term.plot(object,
##           what = c("mu", "sigma", "nu", "tau"),
##           parameter= NULL,
##           data = NULL,
##           envir = environment(formula(object)),
##           partial.resid = FALSE,
##           rug = FALSE,
##           terms = NULL,
##           se = TRUE,
##           ylim = c("common", "free"),
##           scheme = c("shaded", "lines"),
##           xlabs = NULL,
##           ylabs = NULL,
##           main = NULL,
##           pages = 0,
##           col.term = "darkred",
##           col.se = "orange",
##           col.shaded = "gray",
##           col.res = "lightblue",
##           col.rug = "gray",
##           lwd.term = 1.5,
##           lty.se = 2,
##           lwd.se = 1,
##           cex.res = 1,
##           pch.res = par("pch"),
##           ask = interactive() && nb.fig < n.tms && .Device != "postscript",
##           use.factor.levels = TRUE,
##           surface.gam = FALSE,
##           polys = NULL,
##           polys.scheme = "topo",...)


## ---------------------------------------------
term.plot(aids1)

term.plot(aids1, 
          partial.resid = TRUE,
          col.res="black",
          col.se="blue",
          lty.se=1,
          lwd.se = 3,
          lwd.term = 2, 
          col.term="red",
          scheme="lines")

term.plot(aids1,
          partial.resid = TRUE,
          rug = TRUE,
          col.rug="gray",
          col.res="black",
          col.term="blue",
          col.shaded="yellow",
          pch=20,
          scheme="shaded")



## ---- eval=FALSE------------------------------
## wp(object = NULL,
##    xvar = NULL,
##    resid = NULL,
##    n.inter = 4,
##    xcut.points = NULL,
##    overlap = 0,
##    xlim.all = 4,
##    xlim.worm = 3.5,
##    show.given = TRUE,
##    line = TRUE,
##    ylim.all = 12 * sqrt(1/length(resid)),
##    ylim.worm = 12 * sqrt(n.inter/length(resid)),
##    cex = 1,
##    cex.lab = 1,
##    pch = 21,
##    bg = "wheat",
##    col = "red",
##    bar.bg = c(num = "light blue"), ...)


## ---------------------------------------------
wp(aids1,ylim.all=4)

wp(aids1,
   ylim.all=4, 
   col="blue", 
   bg="red")
title(main = "(title)", 
      sub = "(subtitle)")

wp(aids1,
   ylim.all=4, 
   col="orange", 
   bg="blue")


wp(aids1,
   ylim.all =4, 
   col="red", 
   bg="gray")


## ---------------------------------------------
counts <- c(18,17,15,20,10,20,25,13,12)
outcome <- gl(3,1,9)
treatment <- gl(3,3)

d.AD <- data.frame(treatment, outcome, counts)

glm.D93 <- glm(counts ~ outcome + treatment, family = poisson(link="log"))

wp(resid=resid(glm.D93, type="pearson"))

#gamlss.D93<- gamlss(counts ~ outcome + treatment, family = PO(mu.link="log"))
#wp(gamlss.D93)


## ---- eval=FALSE------------------------------
## ## GAIC functions
## AIC(object, ...,
##     k = 2,
##     c = FALSE)
## 
## GAIC(object, ...,
##      k = 2,
##      c = FALSE)
## 
## GAIC.table(object, ...,
##            k = c(2, 3.84,round(log(length(object$y)), 2)),
##            text.to.show=NULL)
## 
## GAIC.scaled(object,..., k = 2, c = FALSE,
##             plot = TRUE,
##             text.cex = 0.7,
##             which = 1,
##             diff.dev = 1000,
##             text.to.show = NULL,
##             col = NULL,
##             horiz = FALSE)
## ## S3 method for class 'gamlss'
## extractAIC(fit,
##            scale,
##            k = 2,
##            c = FALSE, ...)


## ---------------------------------------------
#GAIC.table(aids1,k=c(0,2,round(log(length(aids$y)),5)))
#aids1$G.deviance#the global deviance k=0
#aids1$aic#the Akaike information criterion k=2
#aids1$sbc#the Bayesian information criterion k=log(length(aids$y))
GAIC.table(aids1,aids2,aids3,k=c(0,2,round(log(length(aids$y)),2)))

#GAIC(aids1, aids2, aids3, k=0)#Deviance of GAMLSS
#GAIC(aids1, aids2, aids3, k=2)#AIC
#GAIC(aids1, aids2, aids3,k=round(log(length(aids$y)),2))#BIC


## ---------------------------------------------
plot(aids1)
#plot(aids2)
#plot(aids3)


## ---------------------------------------------
ajusteDis<- fitDist(y,data=aids, 
        k=2,
        type="counts",
        trace=FALSE)
summary(ajusteDis)


## ---------------------------------------------
#escolherD <- chooseDist(aids1, type="counts")
#save.image("escolherD.Rdata")
load("escolherD.Rdata")
#Beta Negative Binomial (BNB) 
#AIC=377.5358
head(escolherD,10)

#which.min(escolherD[,1])#AIC
#which.min(escolherD[,2])#k=3.84
#which.min(escolherD[,3])#BIC
escolherD[which.min(escolherD[,1]),1:3]


## ---------------------------------------------
BS <- function (mu.link ="log", sigma.link="log") {

mstats <- checklink("mu.link","BS",substitute(mu.link),c("inverse", "log", "identity", "own"))
dstats <- checklink("sigma.link","BS",substitute(sigma.link),c("inverse", "log", "identity", "own"))

# fitting information
structure(
list( family 		= c("BS","Birnbaum-Saunders"),
	parameters 		= list(mu=TRUE,sigma=TRUE),
	nopar 		= 2,
	type 			= "Continuous",

	mu.link 		= as.character(substitute(mu.link)),
	sigma.link 		= as.character(substitute(sigma.link)),

	mu.linkfun 		= mstats$linkfun,
	sigma.linkfun	= dstats$linkfun,

	mu.linkinv 		= mstats$linkinv,
	sigma.linkinv 	= dstats$linkinv,

	mu.dr 		= mstats$mu.eta,
	sigma.dr 		= dstats$mu.eta,

  dldm    = function() (1/mu^3)*(y/sigma + sigma/y -2) - 1/mu, #first derivative l w/r to mu
  d2ldm2  = function() -2/mu^2, 						  #E(second derivative l w/r to mu) 
##d2ldm2  = function() {d2ldm2<- eval.parent(expression(-dldp^2)) },#Aproximation								

  dldd    = function() 1/(y+sigma) - 1/(2*sigma)-(1/(2*mu^2))*(1/y - y/sigma^2), ##first derivative l w/r to sigma 
d2ldd2  = function() {d2ldd2 <- eval.parent(expression(-dldp^2))},#Aproximation E(second derivative l w/r to sigma)
	
  d2ldmdd = function() -(2+mu^2)/(mu^3*sigma),	#E(second derivative l w/r to sigma)
##d2ldmdd = function() {d2ldmdd<- eval.parent(expression(-dldp^2))},#Aproximation E(second derivative l w/r to mu and sigma)

G.dev.incr = function(y,mu,sigma,...) GD<- -2*dBS(y, mu, sigma, log = TRUE), 
rqres	     = expression(rqres(pfun="pBS", type="Continuous",y=y, mu=mu, sigma=sigma)),

	mu.initial 		= expression({ mu <- (y+mean(y))/2 }),
	sigma.initial 	= expression({ sigma <- rep(sd(y),length(y))}),

	mu.valid 		= function(mu) all(mu > 0),
	sigma.valid 	= function(sigma) all(sigma > 0),
	y.valid 		= function(y) all(y > 0) ),
	
# the class definition
class = c("gamlss.family","family"))
}

#	the pdf function
dBS<- function(y, mu = 1, sigma = 1, log = FALSE) 
{
    at <- (1/mu) * ( sqrt(y/sigma) - sqrt(sigma/y) )
    At <- y^(-3/2) * (y + sigma)/( 2 * mu * sqrt(sigma) )
    density  <- dnorm(at, 0, 1)*At

    if (log == TRUE) { 
      density <- log(density)
    }
    return(density)
}

#	the cdf function
pBS<- function(	q, mu = 1, sigma = 1, lower.tail = TRUE, log.p = FALSE) 
{

	if (any(mu <= 0)) {
        stop("mu must be positive")
    	}

	if (any(sigma <= 0)) {
        stop("sigma must be positive")
	}

    cumulative <- function(value, a, b) {
    integration<- integrate(dBS, lower = 0, upper = value, mu = a, sigma = b)$value
    return(integration)
    }

    cdf <- mapply(cumulative, q, a = mu, b = sigma) 
		
    if (lower.tail == FALSE) {
        cdf <- (1 - cdf)
    }
    if (log.p == TRUE) {
        cdf <- log(cdf)
    }
    return(cdf)
}

#	 the inverse cdf function
qBS<- function(p, mu = 1, sigma = 1, lower.tail = TRUE, log.p = FALSE) 
{
   if (any(mu <= 0)) {
        stop("mu must be positive")
    }
    if (any(sigma <= 0)) {
        stop("sigma must be positive")
    }
    if (log.p == TRUE) {
        p <- log(p)
    }
    if (lower.tail == FALSE) {
        p <- (1 - p)
    }
    q <- sigma * (    (    (mu * qnorm(p, 0, 1)/2) + sqrt(   ((mu * qnorm(p, 0, 1)/2)^2) + 1)    )      )^2
    return(q)

}

#	the random number generation function
 rBS<- function(n, mu = 1, sigma = 1) 
{
    if (any(mu <= 0)) 
        stop("mu must be positive")
    if (any(sigma <= 0)) 
        stop("sigma must be positive")

    z <- rnorm(n, 0, 1)
    t <- sigma*( 0.5*mu*z  +   sqrt( (0.5*mu*z)^2 +1 )  )^2     #sigma * (1 + (((mu^2) * (z^2))/2) + (mu * z * sqrt((((mu^2) * (z^2))/4) + 1)))
    return(t)
}


## ---------------------------------------------
tempos <-  c(11, 11, 12, 13, 14, 14, 14, 14, 14, 15, 15, 15, 15, 16, 16, 16, 16, 16, 17,
       17, 17, 17, 18, 18, 18, 18, 19, 19, 19, 20, 20, 20, 20, 20, 20, 20, 20, 21, 
       21, 21, 21, 22, 22, 22, 23, 23, 23, 23, 24, 24, 24, 24, 24, 25, 25, 25, 25, 
       25, 25, 26, 26, 26, 26, 26, 27, 27, 27, 27, 27, 28, 28, 29, 29, 29, 29, 29, 
       30, 30, 31, 31, 31, 31, 31, 31, 31, 32, 32, 32, 33, 33, 34, 34, 34, 34, 35, 
       36, 36, 37, 37, 39, 39, 40, 40, 40, 41, 41, 42, 43, 43, 44, 44, 44, 46, 46, 
       46, 47, 47, 48, 50, 50, 51, 52, 53, 54, 55, 56, 57, 59, 61, 66, 70, 89, 97)

dados<- data.frame(t=tempos)

ajuste_tempos<- gamlss(t~1,
                       family = BS(mu.link = "log",sigma.link = "log"),
                       data=dados, trace=FALSE)

fitted(ajuste_tempos,"mu")[1]
fitted(ajuste_tempos,"sigma")[1]

summary(ajuste_tempos)


## ---------------------------------------------
BS()
show.link(family = "BS")

pdf.plot(family = BS(), 
         mu=c(0.1,0.5,0.7,0.9,1.5,2), #alpha
         sigma=1, #beta
         min=0.01, max=5, 
         col=2,lwd=2)


## ---------------------------------------------
SICHEL()# gives information about the default links for the  Sichel distribution 
show.link(family = "SICHEL")

#plot the pdf using plot 
pdf.plot(family = SICHEL(), mu=1, sigma=1,nu=-2:1, min=0, max=8, col=2,lwd=2)


## ---- eval=FALSE------------------------------
## histDist(y, family = NO,
##          freq = NULL,
##          density = FALSE,
##          nbins = 10,
##          xlim = NULL,
##          ylim = NULL,
##          main = NULL,
##          xlab = NULL,
##          ylab = NULL,
##          data = NULL,
##          col.hist = "gray",
##          border.hist = "blue",
##          fg.hist = rainbow(12)[9],
##          line.wd = 2,
##          line.ty = c(1, 2),
##          line.col = c(2, 3),
##          col.main = "blue4",
##          col.lab = "blue4",
##          col.axis = "blue", ...)


## ---- eval=FALSE------------------------------
## gamlssML(formula, family = NO,
##          weights = NULL, mu.start = NULL,
##          sigma.start = NULL,
##          nu.start = NULL,
##          tau.start = NULL,
##          mu.fix = FALSE,
##          sigma.fix = FALSE,
##          nu.fix = FALSE,
##          tau.fix = FALSE,
##          data, start.from = NULL, ...)


## ---------------------------------------------
histDist(t, family = "BS",
         data=dados,
         col.hist="green",
         border.hist = "black",
         line.col = "purple",
         line.ty = 1,
         line.wd = 4,
         )


## ---------------------------------------------
y <- c(0,1,2,3,4,6,7,8,10,11)
freq <- c(33,12,5,6,5,2,2,2,1,2) 
histDist(y, "NBI", freq=freq)


## ---- eval=FALSE------------------------------
## stepGAIC(object,
##          scope,
##          direction = c("both", "backward", "forward"),
##          trace = T,
##          keep = NULL,
##          steps = 1000,
##          scale = 0,
##          what = c("mu", "sigma", "nu", "tau"),
##          parameter= NULL, k = 2,
##          parallel = c("no", "multicore", "snow"),
##          ncpus = 1L,
##          cl = NULL, ...)
## 
## stepGAICAll.A(object,
##               scope = NULL,
##               sigma.scope = NULL,
##               nu.scope = NULL,
##               tau.scope = NULL,
##               mu.try = TRUE,
##               sigma.try = TRUE,
##               nu.try = TRUE,
##               tau.try = TRUE,
##               parallel = c("no", "multicore", "snow"),
##               ncpus = 1L,
##               cl = NULL,  ...)
## 
## 
## stepGAICAll.B(object,
##               scope,
##               direction = c("both", "backward", "forward"),
##               trace = T,
##               keep = NULL,
##               steps = 1000,
##               scale = 0,
##               k = 2,
##               parallel = c("no", "multicore", "snow"),
##               ncpus = 1L,
##               cl = NULL, ...)
## 
## drop1All(object,
##          scope,
##          test = c("Chisq", "none"),
##          k = 2,
##          sorted = FALSE,
##          trace = FALSE,
##          parallel = c("no", "multicore", "snow"),
##          ncpus = 1L, cl = NULL, ...)
## 
## add1All(object,
##         scope,
##         test = c("Chisq", "none"),
##         k = 2,
##         sorted = FALSE,
##         trace = FALSE,
##         parallel = c("no", "multicore", "snow"), ncpus = 1L, cl = NULL, ...)


## ----echo=TRUE, results="hide"----------------
example(birthwt)


## ---------------------------------------------
birthwt.gamlss <- gamlss(low ~ ., 
                         family = BI(mu.link="logit"), 
                         data = bwt)

birthwt.step <- stepGAIC(birthwt.gamlss, 
                         trace = TRUE,
                         k=2)


## ---- eval=FALSE------------------------------
## plotSimpleGamlss(y, x,
##                  model = NULL,
##                  formula = NULL,
##                  data = NULL,
##                  family = NULL,
##                  val = NULL,
##                  N = 1000,
##                  x.val = quantile(x),
##                  ylim = c(min(y), max(y)),
##                  xlim = c(min(x), max(x)),
##                  ...)
## 


## ---------------------------------------------
library(gamlss.util)
library(colorspace)

LO()
show.link(family="LO")

m1 <- gamlss(y~pb(x), 
             sigma.fo=~pb(x), 
             data=abdom, 
             family=LO, 
             trace=FALSE)

plotSimpleGamlss(y,x, 
                 model=m1,   
                 data=abdom, 
                 x.val=seq(15, 40, 5),
                 ylim=c(0, 450), 
                 xlim=c(5, 45),
                 cols=heat_hcl(100))


## ---------------------------------------------
example(gen.Family)


## ---------------------------------------------
example(momentSK)


## ---- eval=FALSE------------------------------
## #install.packages("gamlss.demo")
## #Based on rpanel
## library(gamlss.demo)
## 
## #The demo for gamlss distributions and smoothing
## gamlss.demo()
## 
## #Examples
## #t family distribution
## demo.TF()
## 
## #Skew Normal Type 1 distribution
## demo.SN1()
## 
## #Box-Cox Power Exponential distribution
## demo.BCPE()
## 
## #Demos for smoothing techniques
## demo.BSplines()
## demo.PSplines()
## demo.interpolateSmo()
## demo.histSmo()
## 
## #Interface for demonstrating the gamlss.family distributions
## demoDist()
## 
## #Demo for local polynomial smoothing
## demoLpolyS()


## ---------------------------------------------
logLik_aids1<- gen.likelihood(aids1)
logLik_aids1(par=coef(aids1))


## ---------------------------------------------
vcov(aids1, type="all")


## ---------------------------------------------
confint(aids1)

